<?php

declare(strict_types=1);

namespace Drupal\captcha_captchetat\Controller;

use Drupal\captcha_captchetat\Helper\ClientHelperInterface;
use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\MessageCommand;
use Drupal\Core\Ajax\PrependCommand;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Flood\FloodInterface;
use Drupal\Core\PageCache\ResponsePolicy\KillSwitch;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Manage "CaptchEtat" captcha generation.
 */
class CaptchaGenerateController extends ControllerBase {

  /**
   * The flood captcha generate event name.
   *
   * @var string
   */
  const FLOOD_CAPTCHA_GENERATE = 'captcha_captchetat.captcha_generate';

  /**
   * The "CaptchEtat" client helper.
   *
   * @var \Drupal\captcha_captchetat\Helper\ClientHelperInterface
   */
  protected ClientHelperInterface $clientHelper;

  /**
   * The flood service.
   *
   * @var \Drupal\Core\Flood\FloodInterface
   */
  protected FloodInterface $flood;

  /**
   * The page cache disabling policy.
   *
   * @var \Drupal\Core\PageCache\ResponsePolicy\KillSwitch
   */
  protected KillSwitch $pageCacheKillSwitch;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $instance = parent::create($container);

    $instance->clientHelper = $container->get('captcha_captchetat.helper.client');
    $instance->flood = $container->get('flood');
    $instance->pageCacheKillSwitch = $container->get('page_cache_kill_switch');

    return $instance;
  }

  /**
   * Generate the captcha.
   *
   * @return \Drupal\Core\Ajax\AjaxResponse
   *   The generated captcha or a Drupal message if an error thrown.
   */
  public function generate(): AjaxResponse {
    // Disable recording of cached pages.
    $this->pageCacheKillSwitch->trigger();

    // In case of service is not available.
    if (!$this->clientHelper->isAvailable()) {
      return $this->getUnreachableResponse();
    }

    $config = $this->config('captcha_captchetat.settings');
    $response = new AjaxResponse();

    // Check if user flooding captcha generation.
    if (!$this->flood->isAllowed(
      self::FLOOD_CAPTCHA_GENERATE,
      $config->get('flood.ip_limit'),
      $config->get('flood.ip_window')
    )) {
      // Log flood control denied.
      $this->getLogger('captcha_captchetat_flood_control')
        ->info('Captcha generation is blocked because of IP based flood prevention.');

      return $this->getFloodedResponse();
    }

    // Register user tentative in flood.
    $this->flood->register(self::FLOOD_CAPTCHA_GENERATE, $config->get('flood.ip_window'));

    $captcha = $this->clientHelper->getCaptcha();

    // In case of service doesn't return captcha.
    if (empty($captcha)) {
      return $this->getUnreachableResponse();
    }

    // Let selector empty to use Drupal AJAX element.
    $response->addCommand(new PrependCommand(
      '.captchetat-widget',
      [
        '#type' => 'inline_template',
        '#template' => $captcha,
      ]
    ));

    return $response;
  }

  /**
   * Get "CaptchEtat" flooded response.
   *
   * @return \Drupal\Core\Ajax\AjaxResponse
   *   The flooded AJAX response.
   */
  private function getFloodedResponse(): AjaxResponse {
    $response = new AjaxResponse();
    $response->addCommand(
      new MessageCommand(
        $this->config('captcha_captchetat.texts')->get('flood'),
        NULL,
        ['type' => 'error']
      )
    );
    return $response;
  }

  /**
   * Get "CaptchEtat" unreachable response.
   *
   * @return \Drupal\Core\Ajax\AjaxResponse
   *   The unreachable AJAX response.
   */
  private function getUnreachableResponse(): AjaxResponse {
    $response = new AjaxResponse();
    $response->addCommand(
      new MessageCommand(
        $this->config('captcha_captchetat.texts')->get('unreachable'),
        NULL,
        ['type' => 'error']
      )
    );
    return $response;
  }

}
