<?php

declare(strict_types=1);

namespace Drupal\captcha_captchetat\Controller;

use Drupal\captcha_captchetat\Client\CaptchaClientInterface;
use Drupal\captcha_captchetat\Helper\ClientHelperInterface;
use Drupal\Core\Cache\CacheableResponse;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\PageCache\ResponsePolicy\KillSwitch;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\Exception\AccessDeniedHttpException;

/**
 * Manage "CaptchEtat" captcha object callback.
 */
class CaptchaObjectController extends ControllerBase {

  /**
   * The "CaptchEtat" client helper.
   *
   * @var \Drupal\captcha_captchetat\Helper\ClientHelperInterface
   */
  protected ClientHelperInterface $clientHelper;

  /**
   * The page cache disabling policy.
   *
   * @var \Drupal\Core\PageCache\ResponsePolicy\KillSwitch
   */
  protected KillSwitch $pageCacheKillSwitch;

  /**
   * The current request.
   *
   * @var \Symfony\Component\HttpFoundation\Request
   */
  protected Request $request;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $instance = parent::create($container);

    $instance->clientHelper = $container->get('captcha_captchetat.helper.client');
    $instance->pageCacheKillSwitch = $container->get('page_cache_kill_switch');
    $instance->request = $container->get('request_stack')->getCurrentRequest();

    return $instance;
  }

  /**
   * Get the "CaptchEtat" object in query parameter.
   *
   * @return \Symfony\Component\HttpFoundation\Response
   *   The "CaptchEtat" object.
   */
  public function get(): Response {
    $objectType = $this->request->query->get('get');

    // Object type parameter is required.
    // By default, deny object type HTML with this controller.
    // Use "CaptchaGenerateController" instead.
    if (
      empty($objectType) ||
      $objectType === CaptchaClientInterface::OBJECT_TYPE_HTML
    ) {
      throw new AccessDeniedHttpException();
    }

    $captchaId = $this->request->query->get('t');
    $captchaType = $this->request->query->get('c');

    // Captcha identifier and type are required for this object types.
    if (
      in_array($objectType, [
        CaptchaClientInterface::OBJECT_TYPE_IMAGE,
        CaptchaClientInterface::OBJECT_TYPE_SCRIPT_INCLUDE,
        CaptchaClientInterface::OBJECT_TYPE_SOUND,
      ]) &&
      (
        empty($captchaId) ||
        empty($captchaType)
      )
    ) {
      throw new AccessDeniedHttpException();
    }

    // Don't cache pages when a captcha identifier is defined.
    $response = new Response();

    if (empty($captchaId)) {
      $response = new CacheableResponse();
      $response->getCacheableMetadata()
        ->addCacheContexts(['url.query_args']);
    }

    $object = $this->clientHelper->getCaptcha(
      CaptchaClientInterface::MODE_FRONTAL,
      $objectType,
      $captchaType,
      $captchaId
    );

    $response->setContent($object);

    if ($object) {
      $response->headers->set(
        'Content-Type',
        $this->getResponseContentType($objectType)
      );

      $response->headers->set(
        'Content-Disposition',
        'attachment; filename="' . $this->getResponseFilename($objectType) . '"'
      );
    }

    return $response;
  }

  /**
   * Get response content type.
   *
   * @param string $objectType
   *   The object type to load.
   *
   * @return string
   *   The object's content type.
   */
  private function getResponseContentType(string $objectType): string {
    return match ($objectType) {
      CaptchaClientInterface::OBJECT_TYPE_LAYOUT_STYLESHEET => 'text/css',
      CaptchaClientInterface::OBJECT_TYPE_SCRIPT_INCLUDE => 'application/javascript',
      CaptchaClientInterface::OBJECT_TYPE_IMAGE => 'image/png',
      CaptchaClientInterface::OBJECT_TYPE_RELOAD_ICON,
      CaptchaClientInterface::OBJECT_TYPE_RELOAD_DISABLED_ICON,
      CaptchaClientInterface::OBJECT_TYPE_SOUND_DISABLED_ICON,
      CaptchaClientInterface::OBJECT_TYPE_SOUND_ICON => 'image/gif',
      CaptchaClientInterface::OBJECT_TYPE_SOUND => 'audio/x-wav',
      default  => 'text/html',
    };
  }

  /**
   * Get response filename.
   *
   * @param string $objectType
   *   The object type to load.
   *
   * @return string
   *   The object's filename.
   */
  private function getResponseFilename(string $objectType): string {
    return $objectType . match ($objectType) {
      CaptchaClientInterface::OBJECT_TYPE_LAYOUT_STYLESHEET => '.css',
      CaptchaClientInterface::OBJECT_TYPE_SCRIPT_INCLUDE => '.js',
      CaptchaClientInterface::OBJECT_TYPE_IMAGE => '.png',
      CaptchaClientInterface::OBJECT_TYPE_RELOAD_ICON,
      CaptchaClientInterface::OBJECT_TYPE_RELOAD_DISABLED_ICON,
      CaptchaClientInterface::OBJECT_TYPE_SOUND_DISABLED_ICON,
      CaptchaClientInterface::OBJECT_TYPE_SOUND_ICON => '.gif',
      CaptchaClientInterface::OBJECT_TYPE_SOUND => '.wav',
      default  => '',
    };
  }

}
